# Copyright 2017 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================

r"""Create TFRecord containing subset of PASCAL detection dataset for test.

Example usage:
    python object_detection/dataset_tools/create_pascal_tf_record.py \
        --output_path=/home/user/pascal.record
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import hashlib
import io
import os

import PIL.Image
import tensorflow.compat.v1 as tf
import tensorflow_datasets as tfds

from object_detection.utils import dataset_util
from object_detection.utils import label_map_util


flags = tf.app.flags
flags.DEFINE_string('output_path', None, 'Path to output TFRecord')
flags.mark_flag_as_required('output_path')
flags.DEFINE_string('label_map_path', 'data/pascal_label_map.pbtxt',
                    'Path to label map proto')
FLAGS = flags.FLAGS


def dict_to_tf_example(data,
                       label_map_dict,
                       ignore_difficult_instances=False,
                       image_subdirectory='JPEGImages'):
  """Convert data dict to tf.Example proto.

  Notice that this function normalizes the bounding box coordinates provided
  by the raw data.

  Args:
    data: dict holding PASCAL dataset element from tensorflow_datasets package
    label_map_dict: A map from string label names to integers ids.

  Returns:
    example: The converted tf.Example.
  """

  img = PIL.Image.fromarray(data["image"].numpy())

  # resize to square to make the things simple for square-input models
  img = img.resize((642,642))
  if False:  # for debug
    img.save("test.jpg")

  img_encoded_io = io.BytesIO()
  img.save(img_encoded_io, format='JPEG')
  img_encoded = img_encoded_io.getvalue()

  key = hashlib.sha256(img_encoded).hexdigest()

  width = int(img.width)
  height = int(img.height)

  filename = data["image/filename"].numpy()

  class_names = [
    'aeroplane','bicycle','bird','boat','bottle','bus','car','cat','chair','cow',
    'diningtable','dog','horse','motorbike','person','pottedplant','sheep','sofa','train','tvmonitor']
  assert len(class_names) == 20
  classes = data['objects']['label'].numpy()
  classes_text = [class_names[idx].encode('utf8') for idx in classes]
  print(classes_text, data['objects']['bbox'])

  example = tf.train.Example(features=tf.train.Features(feature={
      'image/height': dataset_util.int64_feature(height),
      'image/width': dataset_util.int64_feature(width),
      'image/filename': dataset_util.bytes_feature(filename),
      'image/source_id': dataset_util.bytes_feature(filename),
      'image/key/sha256': dataset_util.bytes_feature(key.encode('utf8')),
      'image/encoded': dataset_util.bytes_feature(img_encoded),
      'image/format': dataset_util.bytes_feature('jpeg'.encode('utf8')),
      'image/object/bbox/xmin': dataset_util.float_list_feature(list(data['objects']['bbox'][:,1].numpy())),
      'image/object/bbox/xmax': dataset_util.float_list_feature(list(data['objects']['bbox'][:,3].numpy())),
      'image/object/bbox/ymin': dataset_util.float_list_feature(list(data['objects']['bbox'][:,0].numpy())),
      'image/object/bbox/ymax': dataset_util.float_list_feature(list(data['objects']['bbox'][:,2].numpy())),
      'image/object/class/text': dataset_util.bytes_list_feature(classes_text),
      'image/object/class/label': dataset_util.int64_list_feature(classes),
      'image/object/difficult': dataset_util.int64_list_feature(data['objects']['is_difficult'].numpy().astype(int)),
      'image/object/truncated': dataset_util.int64_list_feature(data['objects']['is_truncated'].numpy().astype(int)),
  }))
  return example


def main(_):
  writer = tf.python_io.TFRecordWriter(FLAGS.output_path)

  label_map_dict = label_map_util.get_label_map_dict(FLAGS.label_map_path)

  # load first 128 element from train split
  ds = tfds.load('voc', split='train[0:128]')
  
  for elm in ds:
    tf_example = dict_to_tf_example(elm, label_map_dict)
    writer.write(tf_example.SerializeToString())

  writer.close()


if __name__ == '__main__':
  tf.app.run()
